@@ -1,10 +1,16 @@
 #pragma once
 
+#include <optional>
 #include <string>
 
+#if __has_include(<nlohmann/json_fwd.hpp>)
+#include <nlohmann/json_fwd.hpp>
+#else
 #include <nlohmann/json.hpp>
+#endif
 
 #include "mtx/identifiers.hpp"
+#include "well-known.hpp"
 
 namespace mtx {
 namespace responses {
@@ -17,11 +23,14 @@ struct Login
         //! An access token for the account.
         //! This access token can then be used to authorize other requests.
         std::string access_token;
-        //! The hostname of the homeserver on which the account has been registered.
-        std::string home_server;
         //! ID of the logged-in device.
         //! Will be the same as the corresponding parameter in the request, if one was specified.
         std::string device_id;
+
+        //! Optional client configuration provided by the server.
+        //! If present, clients SHOULD use the provided object to reconfigure themselves,
+        //! optionally validating the URLs within.
+        std::optional<WellKnown> well_known;
 };
 
 void
