@@ -22,6 +22,12 @@ from_json(const json &obj, Audio &content)
 
         if (obj.find("info") != obj.end())
                 content.info = obj.at("info").get<common::AudioInfo>();
+
+        if (obj.find("file") != obj.end())
+                content.file = obj.at("file").get<crypto::EncryptedFile>();
+
+        if (obj.count("m.relates_to") != 0)
+                content.relates_to = obj.at("m.relates_to").get<common::RelatesTo>();
 }
 
 void
@@ -31,6 +37,12 @@ to_json(json &obj, const Audio &content)
         obj["body"]    = content.body;
         obj["url"]     = content.url;
         obj["info"]    = content.info;
+
+        if (content.file)
+                obj["file"] = content.file.value();
+
+        if (!content.relates_to.in_reply_to.event_id.empty())
+                obj["m.relates_to"] = content.relates_to;
 }
 
 } // namespace msg
