@@ -25,6 +25,12 @@ from_json(const json &obj, File &content)
 
         if (obj.find("info") != obj.end())
                 content.info = obj.at("info").get<common::FileInfo>();
+
+        if (obj.find("file") != obj.end())
+                content.file = obj.at("file").get<crypto::EncryptedFile>();
+
+        if (obj.count("m.relates_to") != 0)
+                content.relates_to = obj.at("m.relates_to").get<common::RelatesTo>();
 }
 
 void
@@ -35,6 +41,12 @@ to_json(json &obj, const File &content)
         obj["filename"] = content.filename;
         obj["url"]      = content.url;
         obj["info"]     = content.info;
+
+        if (content.file)
+                obj["file"] = content.file.value();
+
+        if (!content.relates_to.in_reply_to.event_id.empty())
+                obj["m.relates_to"] = content.relates_to;
 }
 
 } // namespace msg
