@@ -24,7 +24,6 @@ using namespace mtx::events::state;
 
 namespace mtx {
 namespace responses {
-
 void
 from_json(const nlohmann::json &obj, GroupId &response)
 {
@@ -44,15 +43,172 @@ from_json(const nlohmann::json &obj, FilterId &response)
 }
 
 namespace utils {
+namespace {
+struct TimelineEventVisitor
+{
+        json operator()(const events::StateEvent<states::Aliases> &aliasEv) const
+        {
+                json j;
+                mtx::events::to_json(j, aliasEv);
+                return j;
+        }
+        json operator()(const events::StateEvent<states::Avatar> &avatarEv) const
+        {
+                json j;
+                mtx::events::to_json(j, avatarEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::CanonicalAlias> &aliasEv) const
+        {
+                json j;
+                mtx::events::to_json(j, aliasEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::Create> &createEv) const
+        {
+                json j;
+                mtx::events::to_json(j, createEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::Encryption> &encEv) const
+        {
+                json j;
+                mtx::events::to_json(j, encEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::GuestAccess> &guestEv) const
+        {
+                json j;
+                mtx::events::to_json(j, guestEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::HistoryVisibility> &histEv) const
+        {
+                json j;
+                mtx::events::to_json(j, histEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::JoinRules> &joinEv) const
+        {
+                json j;
+                mtx::events::to_json(j, joinEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::Member> &membEv) const
+        {
+                json j;
+                mtx::events::to_json(j, membEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::Name> &nameEv) const
+        {
+                json j;
+                mtx::events::to_json(j, nameEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::PinnedEvents> &pinEv) const
+        {
+                json j;
+                mtx::events::to_json(j, pinEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::PowerLevels> &powEv) const
+        {
+                json j;
+                mtx::events::to_json(j, powEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::Tombstone> &tombEv) const
+        {
+                json j;
+                mtx::events::to_json(j, tombEv);
+                return j;
+        };
+        json operator()(const events::StateEvent<states::Topic> &topicEv) const
+        {
+                json j;
+                mtx::events::to_json(j, topicEv);
+                return j;
+        };
+        json operator()(const events::EncryptedEvent<msgs::Encrypted> &encEv) const
+        {
+                json j;
+                mtx::events::to_json(j, encEv);
+                return j;
+        };
+        json operator()(const events::RedactionEvent<msgs::Redaction> &redEv) const
+        {
+                json j;
+                mtx::events::to_json(j, redEv);
+                return j;
+        };
+        json operator()(const events::Sticker &stickEv) const
+        {
+                json j;
+                mtx::events::to_json(j, stickEv);
+                return j;
+        };
+        json operator()(const events::RoomEvent<msgs::Redacted> &redEv) const
+        {
+                json j;
+                mtx::events::to_json(j, redEv);
+                return j;
+        };
+        json operator()(const events::RoomEvent<msgs::Audio> &audioEv) const
+        {
+                json j;
+                mtx::events::to_json(j, audioEv);
+                return j;
+        };
+        json operator()(const events::RoomEvent<msgs::Emote> &emoteEv) const
+        {
+                json j;
+                mtx::events::to_json(j, emoteEv);
+                return j;
+        };
+        json operator()(const events::RoomEvent<msgs::File> &fileEv) const
+        {
+                json j;
+                mtx::events::to_json(j, fileEv);
+                return j;
+        };
+        json operator()(const events::RoomEvent<msgs::Image> &imageEv) const
+        {
+                json j;
+                mtx::events::to_json(j, imageEv);
+                return j;
+        };
+        // TODO: json operator()(const events::RoomEvent<msgs::Location> &locEv) const { json j;
+        // mtx::events::to_json(j, locEv); return j;};
+        json operator()(const events::RoomEvent<msgs::Notice> &noticeEv) const
+        {
+                json j;
+                mtx::events::to_json(j, noticeEv);
+                return j;
+        };
+        json operator()(const events::RoomEvent<msgs::Text> &textEv) const
+        {
+                json j;
+                mtx::events::to_json(j, textEv);
+                return j;
+        };
+        json operator()(const events::RoomEvent<msgs::Video> &videoEv) const
+        {
+                json j;
+                mtx::events::to_json(j, videoEv);
+                return j;
+        };
+};
+}
 
-inline void
+void
 log_error(json::exception &err, const json &event)
 {
         std::cout << err.what() << std::endl;
         std::cout << event.dump(2) << std::endl;
 }
 
-inline void
+void
 log_error(std::string err, const json &event)
 {
         std::cout << err << std::endl;
@@ -79,6 +235,20 @@ parse_room_account_data_events(
                         }
                         break;
                 }
+                case events::EventType::PushRules: {
+                        try {
+                                container.emplace_back(events::Event<pushrules::GlobalRuleset>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+                        break;
+                }
+                case events::EventType::KeyVerificationCancel:
+                case events::EventType::KeyVerificationRequest:
+                case events::EventType::KeyVerificationStart:
+                case events::EventType::KeyVerificationAccept:
+                case events::EventType::KeyVerificationKey:
+                case events::EventType::KeyVerificationMac:
                 case events::EventType::RoomKeyRequest:
                 case events::EventType::RoomAliases:
                 case events::EventType::RoomAvatar:
@@ -93,6 +263,7 @@ parse_room_account_data_events(
                 case events::EventType::RoomMessage:
                 case events::EventType::RoomName:
                 case events::EventType::RoomPowerLevels:
+                case events::EventType::RoomTombstone:
                 case events::EventType::RoomTopic:
                 case events::EventType::RoomRedaction:
                 case events::EventType::RoomPinnedEvents:
@@ -103,6 +274,14 @@ parse_room_account_data_events(
         }
 }
 
+void
+compose_timeline_events(json &events,
+                        const std::vector<mtx::events::collections::TimelineEvents> &container)
+{
+        const auto c = container.at(0);
+        events       = std::visit(TimelineEventVisitor(), c);
+}
+
 void
 parse_timeline_events(const json &events,
                       std::vector<mtx::events::collections::TimelineEvents> &container)
@@ -233,6 +412,15 @@ parse_timeline_events(const json &events,
 
                         break;
                 }
+                case events::EventType::RoomTombstone: {
+                        try {
+                                container.emplace_back(events::StateEvent<Tombstone>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                }
                 case events::EventType::RoomTopic: {
                         try {
                                 container.emplace_back(events::StateEvent<Topic>(e));
@@ -358,7 +546,14 @@ parse_timeline_events(const json &events,
                 case events::EventType::RoomPinnedEvents:
                 case events::EventType::RoomKeyRequest: // Not part of the timeline
                 case events::EventType::Tag:            // Not part of the timeline or state
+                case events::EventType::PushRules:      // Not part of the timeline or state
                 case events::EventType::Unsupported:
+                case events::EventType::KeyVerificationCancel:
+                case events::EventType::KeyVerificationRequest:
+                case events::EventType::KeyVerificationStart:
+                case events::EventType::KeyVerificationAccept:
+                case events::EventType::KeyVerificationKey:
+                case events::EventType::KeyVerificationMac:
                         continue;
                 }
         }
@@ -474,6 +669,15 @@ parse_state_events(const json &events,
 
                         break;
                 }
+                case events::EventType::RoomTombstone: {
+                        try {
+                                container.emplace_back(events::StateEvent<Tombstone>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                }
                 case events::EventType::RoomTopic: {
                         try {
                                 container.emplace_back(events::StateEvent<Topic>(e));
@@ -489,8 +693,15 @@ parse_state_events(const json &events,
                 case events::EventType::RoomMessage:
                 case events::EventType::RoomPinnedEvents:
                 case events::EventType::RoomRedaction:
-                case events::EventType::Tag: // Not part of the timeline or state
+                case events::EventType::Tag:       // Not part of the timeline or state
+                case events::EventType::PushRules: // Not part of the timeline or state
                 case events::EventType::Unsupported:
+                case events::EventType::KeyVerificationCancel:
+                case events::EventType::KeyVerificationRequest:
+                case events::EventType::KeyVerificationStart:
+                case events::EventType::KeyVerificationAccept:
+                case events::EventType::KeyVerificationKey:
+                case events::EventType::KeyVerificationMac:
                         continue;
                 }
         }
@@ -597,6 +808,15 @@ parse_stripped_events(const json &events,
 
                         break;
                 }
+                case events::EventType::RoomTombstone: {
+                        try {
+                                container.emplace_back(events::StrippedEvent<Tombstone>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                }
                 case events::EventType::RoomTopic: {
                         try {
                                 container.emplace_back(events::StrippedEvent<Topic>(e));
@@ -613,8 +833,15 @@ parse_stripped_events(const json &events,
                 case events::EventType::RoomRedaction:
                 case events::EventType::RoomKeyRequest: // Not part of the timeline or state
                 case events::EventType::RoomPinnedEvents:
-                case events::EventType::Tag: // Not part of the timeline or state
+                case events::EventType::Tag:       // Not part of the timeline or state
+                case events::EventType::PushRules: // Not part of the timeline or state
                 case events::EventType::Unsupported:
+                case events::EventType::KeyVerificationCancel:
+                case events::EventType::KeyVerificationRequest:
+                case events::EventType::KeyVerificationStart:
+                case events::EventType::KeyVerificationAccept:
+                case events::EventType::KeyVerificationKey:
+                case events::EventType::KeyVerificationMac:
                         continue;
                 }
         }
