@@ -51,6 +51,7 @@ TEST(Events, Conversions)
         EXPECT_EQ("m.room.name", ns::to_string(ns::EventType::RoomName));
         EXPECT_EQ("m.room.power_levels", ns::to_string(ns::EventType::RoomPowerLevels));
         EXPECT_EQ("m.room.topic", ns::to_string(ns::EventType::RoomTopic));
+        EXPECT_EQ("m.room.tombstone", ns::to_string(ns::EventType::RoomTombstone));
         EXPECT_EQ("m.room.redaction", ns::to_string(ns::EventType::RoomRedaction));
         EXPECT_EQ("m.room.pinned_events", ns::to_string(ns::EventType::RoomPinnedEvents));
         EXPECT_EQ("m.tag", ns::to_string(ns::EventType::Tag));
@@ -179,6 +180,41 @@ TEST(StateEvents, Create)
         EXPECT_EQ(event.origin_server_ts, 1506761923948L);
         EXPECT_EQ(event.state_key, "");
         EXPECT_EQ(event.content.creator, "@mujx:matrix.org");
+
+        json example_from_spec = R"({
+            "content": {
+                "creator": "@example:example.org",
+                "m.federate": true,
+                "predecessor": {
+                    "event_id": "$something:example.org",
+                    "room_id": "!oldroom:example.org"
+                },
+                "room_version": "1"
+            },
+            "event_id": "$143273582443PhrSn:example.org",
+            "origin_server_ts": 1432735824653,
+            "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
+            "sender": "@example:example.org",
+            "state_key": "",
+            "type": "m.room.create",
+            "unsigned": {
+                "age": 1234
+            }
+        })"_json;
+
+        event = example_from_spec;
+
+        EXPECT_EQ(event.type, ns::EventType::RoomCreate);
+        EXPECT_EQ(event.event_id, "$143273582443PhrSn:example.org");
+        EXPECT_EQ(event.sender, "@example:example.org");
+        EXPECT_EQ(event.unsigned_data.age, 1234);
+        EXPECT_EQ(event.origin_server_ts, 1432735824653L);
+        EXPECT_EQ(event.state_key, "");
+        EXPECT_EQ(event.content.creator, "@example:example.org");
+        EXPECT_EQ(event.content.federate, true);
+        EXPECT_EQ(event.content.room_version, "1");
+        EXPECT_EQ(event.content.predecessor->room_id, "!oldroom:example.org");
+        EXPECT_EQ(event.content.predecessor->event_id, "$something:example.org");
 }
 
 TEST(StateEvents, GuestAccess)
@@ -294,6 +330,56 @@ TEST(StateEvents, JoinRules)
         EXPECT_EQ(event.origin_server_ts, 1506761924018L);
         EXPECT_EQ(event.state_key, "");
         EXPECT_EQ(event.content.join_rule, ns::state::JoinRule::Invite);
+
+        EXPECT_EQ(data, json(event));
+
+        data = R"({
+          "origin_server_ts": 1506761924018,
+          "sender": "@mujx:matrix.org",
+          "event_id": "$15067619241414401ASocy:matrix.org",
+          "unsigned": {
+            "age": 3715756273
+	  },
+          "state_key": "",
+          "content": {
+            "join_rule": "public"
+	  },
+          "type": "m.room.join_rules"
+        })"_json;
+
+        EXPECT_EQ(data, json(ns::StateEvent<ns::state::JoinRules>(data)));
+
+        data = R"({
+          "origin_server_ts": 1506761924018,
+          "sender": "@mujx:matrix.org",
+          "event_id": "$15067619241414401ASocy:matrix.org",
+          "unsigned": {
+            "age": 3715756273
+	  },
+          "state_key": "",
+          "content": {
+            "join_rule": "knock"
+	  },
+          "type": "m.room.join_rules"
+        })"_json;
+
+        EXPECT_EQ(data, json(ns::StateEvent<ns::state::JoinRules>(data)));
+
+        data = R"({
+          "origin_server_ts": 1506761924018,
+          "sender": "@mujx:matrix.org",
+          "event_id": "$15067619241414401ASocy:matrix.org",
+          "unsigned": {
+            "age": 3715756273
+	  },
+          "state_key": "",
+          "content": {
+            "join_rule": "private"
+	  },
+          "type": "m.room.join_rules"
+        })"_json;
+
+        EXPECT_EQ(data, json(ns::StateEvent<ns::state::JoinRules>(data)));
 }
 
 TEST(StateEvents, Member)
@@ -512,6 +598,37 @@ TEST(StateEvents, PowerLevels)
         EXPECT_EQ(event.content.user_level("@not:matrix.org"), event.content.users_default);
 }
 
+TEST(StateEvents, Tombstone)
+{
+        json data = R"({
+            "content": {
+                "body": "This room has been replaced",
+                "replacement_room": "!newroom:example.org"
+            },
+            "event_id": "$143273582443PhrSn:example.org",
+            "origin_server_ts": 1432735824653,
+            "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
+            "sender": "@example:example.org",
+            "state_key": "",
+            "type": "m.room.tombstone",
+            "unsigned": {
+                "age": 1234
+            }
+        })"_json;
+
+        ns::StateEvent<ns::state::Tombstone> event = data;
+
+        EXPECT_EQ(event.type, ns::EventType::RoomTombstone);
+        EXPECT_EQ(event.event_id, "$143273582443PhrSn:example.org");
+        EXPECT_EQ(event.room_id, "!jEsUZKDJdhlrceRyVU:example.org");
+        EXPECT_EQ(event.sender, "@example:example.org");
+        EXPECT_EQ(event.origin_server_ts, 1432735824653);
+        EXPECT_EQ(event.unsigned_data.age, 1234);
+        EXPECT_EQ(event.state_key, "");
+        EXPECT_EQ(event.content.body, "This room has been replaced");
+        EXPECT_EQ(event.content.replacement_room, "!newroom:example.org");
+}
+
 TEST(StateEvents, Topic)
 {
         json data = R"({
@@ -659,6 +776,172 @@ TEST(RoomEvents, Encrypted)
           "\"session_id\":\"/bHcdWPHsJLFd8dkyvG0n7q/RTDmfBIc+gC4laHJCQQ\"}");
 }
 
+TEST(ToDevice, KeyVerificationRequest)
+{
+        json request_data = R"({
+    "content": {
+        "from_device": "AliceDevice2",
+        "methods": [
+            "m.sas.v1"
+        ],
+        "timestamp": 1559598944869,
+        "transaction_id": "S0meUniqueAndOpaqueString"
+    },
+    "type": "m.key.verification.request"
+})"_json;
+
+        ns::Event<ns::msg::KeyVerificationRequest> event = request_data;
+        auto keyEvent                                    = event.content;
+        EXPECT_EQ(keyEvent.from_device, "AliceDevice2");
+        EXPECT_EQ(event.type, mtx::events::EventType::KeyVerificationRequest);
+        EXPECT_EQ(keyEvent.transaction_id, "S0meUniqueAndOpaqueString");
+        EXPECT_EQ(keyEvent.methods[0], ns::msg::VerificationMethods::SASv1);
+        EXPECT_EQ(keyEvent.timestamp, 1559598944869);
+        EXPECT_EQ(request_data.dump(), json(event).dump());
+}
+
+TEST(ToDevice, KeyVerificationStart)
+{
+        json request_data = R"({
+    "content": {
+        "from_device": "BobDevice1",
+        "hashes": [
+            "sha256"
+        ],
+        "key_agreement_protocols": [
+            "curve25519"
+        ],
+        "message_authentication_codes": [
+            "hkdf-hmac-sha256"
+        ],
+        "method": "m.sas.v1",
+        "short_authentication_string": [
+            "decimal",
+            "emoji",
+            "some-random-invalid-method"
+        ],
+        "transaction_id": "S0meUniqueAndOpaqueString"
+    },
+    "type": "m.key.verification.start"
+})"_json;
+
+        ns::Event<ns::msg::KeyVerificationStart> event = request_data;
+        auto keyEvent                                  = event.content;
+        EXPECT_EQ(keyEvent.from_device, "BobDevice1");
+        EXPECT_EQ(keyEvent.hashes[0], "sha256");
+        EXPECT_EQ(keyEvent.key_agreement_protocols[0], "curve25519");
+        EXPECT_EQ(keyEvent.message_authentication_codes[0], "hkdf-hmac-sha256");
+        EXPECT_EQ(keyEvent.short_authentication_string[0], ns::msg::SASMethods::Decimal);
+        EXPECT_EQ(keyEvent.short_authentication_string[1], ns::msg::SASMethods::Emoji);
+        EXPECT_EQ(keyEvent.short_authentication_string[2], ns::msg::SASMethods::Unsupported);
+        EXPECT_EQ(event.type, mtx::events::EventType::KeyVerificationStart);
+        EXPECT_EQ(keyEvent.transaction_id, "S0meUniqueAndOpaqueString");
+        EXPECT_EQ(keyEvent.method, ns::msg::VerificationMethods::SASv1);
+        // The incoming and outgoing JSON will not match due to the Unsupported SASMethod in the
+        // request_data, so no point in comparing the dump of both for equality.
+}
+
+TEST(ToDevice, KeyVerificationAccept)
+{
+        json request_data = R"({
+    "content": {
+        "commitment": "fQpGIW1Snz+pwLZu6sTy2aHy/DYWWTspTJRPyNp0PKkymfIsNffysMl6ObMMFdIJhk6g6pwlIqZ54rxo8SLmAg",
+        "hash": "sha256",
+        "key_agreement_protocol": "curve25519",
+        "message_authentication_code": "hkdf-hmac-sha256",
+        "method": "m.sas.v1",
+        "short_authentication_string": [
+            "decimal",
+            "emoji"
+        ],
+        "transaction_id": "S0meUniqueAndOpaqueString"
+    },
+    "type": "m.key.verification.accept"
+})"_json;
+
+        ns::Event<ns::msg::KeyVerificationAccept> event = request_data;
+        auto keyEvent                                   = event.content;
+        EXPECT_EQ(event.type, mtx::events::EventType::KeyVerificationAccept);
+        EXPECT_EQ(
+          keyEvent.commitment,
+          "fQpGIW1Snz+pwLZu6sTy2aHy/DYWWTspTJRPyNp0PKkymfIsNffysMl6ObMMFdIJhk6g6pwlIqZ54rxo8SLmAg");
+        EXPECT_EQ(keyEvent.hash, "sha256");
+        EXPECT_EQ(keyEvent.key_agreement_protocol, "curve25519");
+        EXPECT_EQ(keyEvent.message_authentication_code, "hkdf-hmac-sha256");
+        EXPECT_EQ(keyEvent.method, ns::msg::VerificationMethods::SASv1);
+        EXPECT_EQ(keyEvent.short_authentication_string[0], ns::msg::SASMethods::Decimal);
+        EXPECT_EQ(keyEvent.short_authentication_string[1], ns::msg::SASMethods::Emoji);
+        EXPECT_EQ(keyEvent.transaction_id, "S0meUniqueAndOpaqueString");
+        EXPECT_EQ(request_data.dump(), json(event).dump());
+}
+
+TEST(ToDevice, KeyVerificationCancel)
+{
+        json request_data = R"({
+    "content": {
+        "code": "m.user",
+        "reason": "User rejected the key verification request",
+        "transaction_id": "S0meUniqueAndOpaqueString"
+    },
+    "type": "m.key.verification.cancel"
+})"_json;
+
+        ns::Event<ns::msg::KeyVerificationCancel> event = request_data;
+        auto keyEvent                                   = event.content;
+        EXPECT_EQ(event.type, mtx::events::EventType::KeyVerificationCancel);
+        EXPECT_EQ(keyEvent.code, "m.user");
+        EXPECT_EQ(keyEvent.reason, "User rejected the key verification request");
+        EXPECT_EQ(keyEvent.transaction_id, "S0meUniqueAndOpaqueString");
+        EXPECT_EQ(request_data.dump(), json(event).dump());
+}
+
+TEST(ToDevice, KeyVerificationKey)
+{
+        json request_data = R"({
+    "content": {
+        "key": "fQpGIW1Snz+pwLZu6sTy2aHy/DYWWTspTJRPyNp0PKkymfIsNffysMl6ObMMFdIJhk6g6pwlIqZ54rxo8SLmAg",
+        "transaction_id": "S0meUniqueAndOpaqueString"
+    },
+    "type": "m.key.verification.key"
+})"_json;
+
+        ns::Event<ns::msg::KeyVerificationKey> event = request_data;
+        auto keyEvent                                = event.content;
+        EXPECT_EQ(event.type, mtx::events::EventType::KeyVerificationKey);
+        EXPECT_EQ(
+          keyEvent.key,
+          "fQpGIW1Snz+pwLZu6sTy2aHy/DYWWTspTJRPyNp0PKkymfIsNffysMl6ObMMFdIJhk6g6pwlIqZ54rxo8SLmAg");
+        EXPECT_EQ(keyEvent.transaction_id, "S0meUniqueAndOpaqueString");
+        EXPECT_EQ(request_data.dump(), json(event).dump());
+}
+
+TEST(ToDevice, KeyVerificationMac)
+{
+        json request_data = R"({
+    "content": {
+        "keys": "2Wptgo4CwmLo/Y8B8qinxApKaCkBG2fjTWB7AbP5Uy+aIbygsSdLOFzvdDjww8zUVKCmI02eP9xtyJxc/cLiBA",
+        "mac": {
+            "ed25519:ABCDEF": "fQpGIW1Snz+pwLZu6sTy2aHy/DYWWTspTJRPyNp0PKkymfIsNffysMl6ObMMFdIJhk6g6pwlIqZ54rxo8SLmAg"
+        },
+        "transaction_id": "S0meUniqueAndOpaqueString"
+    },
+    "type": "m.key.verification.mac"
+})"_json;
+
+        ns::Event<ns::msg::KeyVerificationMac> event = request_data;
+        auto keyEvent                                = event.content;
+        EXPECT_EQ(event.type, mtx::events::EventType::KeyVerificationMac);
+        EXPECT_EQ(
+          keyEvent.keys,
+          "2Wptgo4CwmLo/Y8B8qinxApKaCkBG2fjTWB7AbP5Uy+aIbygsSdLOFzvdDjww8zUVKCmI02eP9xtyJxc/cLiBA");
+        EXPECT_EQ(keyEvent.mac.count("ed25519:ABCDEF"), 1);
+        EXPECT_EQ(
+          keyEvent.mac.at("ed25519:ABCDEF"),
+          "fQpGIW1Snz+pwLZu6sTy2aHy/DYWWTspTJRPyNp0PKkymfIsNffysMl6ObMMFdIJhk6g6pwlIqZ54rxo8SLmAg");
+        EXPECT_EQ(keyEvent.transaction_id, "S0meUniqueAndOpaqueString");
+        EXPECT_EQ(request_data.dump(), json(event).dump());
+}
+
 TEST(ToDevice, KeyRequest)
 {
         json request_data = R"({
@@ -736,7 +1019,7 @@ TEST(Collection, Events)
 
         mtx::events::collections::TimelineEvent event = data;
 
-        ASSERT_TRUE(boost::get<ns::StateEvent<ns::state::Aliases>>(&event.data) != nullptr);
+        ASSERT_TRUE(std::get_if<ns::StateEvent<ns::state::Aliases>>(&event.data) != nullptr);
 }
 
 TEST(RoomAccountData, Tag)
